@@ -111,17 +111,17 @@ handleTimeouts(void)
   struct timeval now;
   GLUTtimer *timer;
 
-  if (__glutTimerList) {
-    GETTIMEOFDAY(&now);
-    while (IS_AT_OR_AFTER(__glutTimerList->timeout, now)) {
-      timer = __glutTimerList;
-      timer->func(timer->value);
-      __glutTimerList = timer->next;
-      timer->next = freeTimerList;
-      freeTimerList = timer;
-      if (!__glutTimerList)
-        break;
-    }
+  /* Assumption is that __glutTimerList is already determined
+     to be non-NULL. */
+  GETTIMEOFDAY(&now);
+  while (IS_AT_OR_AFTER(__glutTimerList->timeout, now)) {
+    timer = __glutTimerList;
+    timer->func(timer->value);
+    __glutTimerList = timer->next;
+    timer->next = freeTimerList;
+    freeTimerList = timer;
+    if (!__glutTimerList)
+      break;
   }
 }
 
@@ -142,12 +142,13 @@ __glutPutOnWorkList(GLUTwindow * window, int workMask)
 void
 __glutPostRedisplay(GLUTwindow * window, int layerMask)
 {
-  int shown = (layerMask == GLUT_REDISPLAY_WORK) ? window->shownState : window->overlay->shownState;
+  int shown = (layerMask & (GLUT_REDISPLAY_WORK | GLUT_REPAIR_WORK)) ?
+    window->shownState : window->overlay->shownState;
 
   /* Post a redisplay if the window is visible (or the
      visibility of the window is unknown, ie. window->visState
      == -1) _and_ the layer is known to be shown. */
-  if (window->visState != 0 && shown)
+  if (window->visState != GLUT_HIDDEN && window->visState != GLUT_FULLY_COVERED && shown)
     __glutPutOnWorkList(window, layerMask);
 }
 
@@ -163,9 +164,9 @@ static GLUTeventParser *eventParserList = NULL;
 
 /* __glutRegisterEventParser allows another module to register
    to intercept X events types not otherwise acted on by the
-   GLUT processEvents routine.  The X Input extension support
-   code uses an event parser for handling X Input extension
-   events.  */
+   GLUT processEventsAndTimeouts routine.  The X Input
+   extension support code uses an event parser for handling X
+   Input extension events.  */
 
 void
 __glutRegisterEventParser(GLUTeventParser * parser)
@@ -175,24 +176,22 @@ __glutRegisterEventParser(GLUTeventParser * parser)
 }
 
 static void
-updateWindowVisibility(GLUTwindow * window, int visState)
+markWindowHidden(GLUTwindow * window)
 {
-  if (window->shownState && visState != window->visState) {
-    if (window->visibility) {
-      window->visState = visState;
+  if (GLUT_HIDDEN != window->visState) {
+    GLUTwindow *child;
+
+    if (window->windowStatus) {
+      window->visState = GLUT_HIDDEN;
       __glutSetWindow(window);
-      window->visibility(visState ?
-        GLUT_VISIBLE : GLUT_NOT_VISIBLE);
+      window->windowStatus(GLUT_HIDDEN);
     }
     /* An unmap is only reported on a single window; its
        descendents need to know they are no longer visible. */
-    if (!visState) {
-      GLUTwindow *child = window->children;
-
-      while (child) {
-        updateWindowVisibility(child, visState);
-        child = child->siblings;
-      }
+    child = window->children;
+    while (child) {
+      markWindowHidden(child);
+      child = child->siblings;
     }
   }
 }
@@ -219,7 +218,7 @@ purgeStaleWindow(Window win)
 }
 
 static void
-processEvents(void)
+processEventsAndTimeouts(void)
 {
   XEvent event, ahead;
   GLUTwindow *window;
@@ -255,6 +254,13 @@ processEvents(void)
           glXWaitX();
           window->reshape(width, height);
           window->forceReshape = False;
+          /* A reshape should be considered like posting a
+             redisplay; this is necessary for the "Mesa
+             glXSwapBuffers to repair damage" hack to operate
+             correctly.  Without it, there's not an initial
+             back buffer render from which to blit from when
+             damage happens to the window. */
+          __glutPostRedisplay(window, GLUT_REDISPLAY_WORK);
         }
       }
       break;
@@ -278,11 +284,9 @@ processEvents(void)
           window = __glutGetWindow(event.xexpose.window);
           if (window) {
             if (window->win == event.xexpose.window) {
-              window->damaged = 1;
-              __glutPostRedisplay(window, GLUT_REDISPLAY_WORK);
+              __glutPostRedisplay(window, GLUT_REPAIR_WORK);
             } else if (window->overlay && window->overlay->win == event.xexpose.window) {
-              __glutPostRedisplay(window, GLUT_OVERLAY_REDISPLAY_WORK);
-              window->overlay->damaged = 1;
+              __glutPostRedisplay(window, GLUT_OVERLAY_REPAIR_WORK);
             }
           }
         }
@@ -437,7 +441,7 @@ processEvents(void)
         case XK_DeleteChar:
           /* The Delete character is really an ASCII key. */
           __glutSetWindow(window);
-          window->keyboard(127, /* ASCII Delete character. */
+          window->keyboard(127,  /* ASCII Delete character. */
             event.xkey.x, event.xkey.y);
           goto skip;
 #endif
@@ -564,19 +568,22 @@ processEvents(void)
              StructureNotify events to receive DestroyNotify. */
           break;
         }
-        updateWindowVisibility(window, 0);
+        markWindowHidden(window);
       }
       break;
     case VisibilityNotify:
       window = __glutGetWindow(event.xvisibility.window);
       if (window) {
-        int visState = (event.xvisibility.state != VisibilityFullyObscured);
+	/* VisibilityUnobscured+1 = GLUT_FULLY_RETAINED,
+	   VisibilityPartiallyObscured+1 = GLUT_PARTIALLY_RETAINED,
+	   VisibilityFullyObscured+1 = GLUT_FULLY_COVERED. */
+        int visState = event.xvisibility.state + 1;
 
         if (visState != window->visState) {
-          if (window->visibility) {
+          if (window->windowStatus) {
             window->visState = visState;
             __glutSetWindow(window);
-            window->visibility(visState ? GLUT_VISIBLE : GLUT_NOT_VISIBLE);
+            window->windowStatus(visState);
           }
         }
       }
@@ -609,6 +616,8 @@ processEvents(void)
       }
       break;
     }
+    if (__glutTimerList)
+      handleTimeouts();
   }
   while (XPending(__glutDisplay));
 }
@@ -617,12 +626,14 @@ static void
 waitForSomething(void)
 {
 #ifdef __vms
-  static struct timeval zerotime = {0};
+  static struct timeval zerotime =
+  {0};
   unsigned int timer_efn;
-#define timer_id 'glut'           /* random :-) number */
+#define timer_id 'glut' /* random :-) number */
   unsigned int wait_mask;
 #else
-  static struct timeval zerotime = {0, 0};
+  static struct timeval zerotime =
+  {0, 0};
   fd_set fds;
 #endif
   struct timeval now, timeout, waittime;
@@ -649,22 +660,22 @@ waitForSomething(void)
   wait_mask = 1 << (__glutConnectionFD & 31);
   if (IS_AFTER(now, timeout)) {
     /* We need an event flag for the timer. */
-    /* XXX The `right' way to do this is to use LIB$GET_EF, but since it needs
-       to be in the same cluster as the EFN for the display, we will have
-       hack it. */
+    /* XXX The `right' way to do this is to use LIB$GET_EF, but
+       since it needs to be in the same cluster as the EFN for
+       the display, we will have hack it. */
     timer_efn = __glutConnectionFD - 1;
-    if ((timer_efn / 32) != (__glutConnectionFD / 32) ) {
+    if ((timer_efn / 32) != (__glutConnectionFD / 32)) {
       timer_efn = __glutConnectionFD + 1;
     }
-    rc = SYS$CLREF (timer_efn);
-    rc = SYS$SETIMR (timer_efn, &timeout, NULL, timer_id, 0);
+    rc = SYS$CLREF(timer_efn);
+    rc = SYS$SETIMR(timer_efn, &timeout, NULL, timer_id, 0);
     wait_mask |= 1 << (timer_efn & 31);
   } else {
     timer_efn = 0;
   }
-  rc = SYS$WFLOR (__glutConnectionFD, wait_mask);
-  if (timer_efn != 0 && SYS$CLREF (timer_efn) == SS$_WASCLR) {
-    rc = SYS$CANTIM (timer_id, PSL$C_USER);
+  rc = SYS$WFLOR(__glutConnectionFD, wait_mask);
+  if (timer_efn != 0 && SYS$CLREF(timer_efn) == SS$_WASCLR) {
+    rc = SYS$CANTIM(timer_id, PSL$C_USER);
   }
   /* XXX There does not seem to be checking of "rc" in the code
      above.  Can any of the SYS$ routines above fail? */
@@ -684,25 +695,30 @@ waitForSomething(void)
     __glutFatalError("select error.");
 #endif /* not vms */
   /* Without considering the cause of select unblocking, check
-     for pending X events *and* then handle any timeouts. We
-     always look for X events even if select returned with 0
-     (indicating a timeout); otherwise we risk starving X event
-     processing by continous timeouts. */
-  while (XPending(__glutDisplay)) {
+     for pending X events and handle any timeouts (by calling
+     processEventsAndTimeouts).  We always look for X events
+     even if select returned with 0 (indicating a timeout);
+     otherwise we risk starving X event processing by continous 
+
+     timeouts. */
+  if (XPending(__glutDisplay)) {
   immediatelyHandleXinput:
-    processEvents();
+    processEventsAndTimeouts();
+  } else {
+    if (__glutTimerList)
+      handleTimeouts();
   }
-  handleTimeouts();
 }
 
 static void
 idleWait(void)
 {
-  while (XPending(__glutDisplay)) {
-    processEvents();
+  if (XPending(__glutDisplay)) {
+    processEventsAndTimeouts();
+  } else {
+    if (__glutTimerList)
+      handleTimeouts();
   }
-  if (__glutTimerList)
-    handleTimeouts();
   /* Make sure idle func still exists! */
   if (__glutIdleFunc)
     __glutIdleFunc();
@@ -739,7 +755,7 @@ processWindowWorkList(GLUTwindow * window)
 
   /* Optimization: most of the time, the work to do is a
      redisplay and not these other types of work.  Check for
-     the following cases as a group to before checking each one 
+     the following cases as a group to before checking each one
      individually one by one. This saves about 25 MIPS
      instructions in the common redisplay only case. */
   if (workMask & (GLUT_EVENT_MASK_WORK | GLUT_DEVICE_MASK_WORK |
@@ -803,6 +819,27 @@ processWindowWorkList(GLUTwindow * window)
             XChangeProperty(__glutDisplay, window->win,
               __glutMotifHints, __glutMotifHints, 32,
               PropModeReplace, (unsigned char *) &hints, 4);
+            if (workMask & GLUT_MAP_WORK) {
+              /* Handle case where glutFullScreen is called
+                 before the first time that the window is mapped.
+                 Some window managers will randomly or
+                 interactively position the window the first time
+                 it is mapped if the window's WM_NORMAL_HINTS
+                 property does not request an explicit position.
+                 We don't want any such window manager interaction
+                 when going fullscreen.  Overwrite the
+                 WM_NORMAL_HINTS property installed by
+                 glutCreateWindow's XSetWMProperties property with
+                 one explicitly requesting a fullscreen window. */
+              XSizeHints hints;
+
+              hints.flags = USPosition | USSize;
+              hints.x = 0;
+              hints.y = 0;
+              hints.width = window->desiredWidth;
+              hints.height = window->desiredHeight;
+              XSetWMNormalHints(__glutDisplay, window->win, &hints);
+            }
           } else {
             XDeleteProperty(__glutDisplay, window->win, __glutMotifHints);
           }
@@ -849,23 +886,31 @@ processWindowWorkList(GLUTwindow * window)
       }
     }
   }
-  if (workMask & (GLUT_REDISPLAY_WORK | GLUT_OVERLAY_REDISPLAY_WORK)) {
+  if (workMask & (GLUT_REDISPLAY_WORK | GLUT_OVERLAY_REDISPLAY_WORK | GLUT_REPAIR_WORK | GLUT_OVERLAY_REPAIR_WORK)) {
     if (window->forceReshape) {
       /* Guarantee that before a display callback is generated
          for a window, a reshape callback must be generated. */
       __glutSetWindow(window);
       window->reshape(window->width, window->height);
       window->forceReshape = False;
+
+      /* Setting the redisplay bit on the first reshape is
+         necessary to make the "Mesa glXSwapBuffers to repair
+         damage" hack operate correctly.  Without indicating a
+         redisplay is necessary, there's not an initial back
+         buffer render from which to blit from when damage
+         happens to the window. */
+      workMask |= GLUT_REDISPLAY_WORK;
     }
     /* The code below is more involved than otherwise necessary
        because it is paranoid about the overlay or entire window
        being removed or destroyed in the course of the callbacks.
-       Notice how the global __glutWindowDamaged is used to
-       record the layers' damage status.  See the code in
-       glutLayerGet for how __glutWindowDamaged is used. The 
-       point is to not have to update the "damaged" field after 
-       the callback since the window (or overlay) may be
-       destroyed (or removed) when the callback returns. */
+       Notice how the global __glutWindowDamaged is used to record
+       the layers' damage status.  See the code in glutLayerGet for
+       how __glutWindowDamaged is used. The  point is to not have to
+       update the "damaged" field after  the callback since the
+       window (or overlay) may be destroyed (or removed) when the
+       callback returns. */
 
     if (window->overlay && window->overlay->display) {
       int num = window->num;
@@ -877,18 +922,27 @@ processWindowWorkList(GLUTwindow * window)
          display callback registered, we simply use the
          standard display callback. */
 
-      if (workMask & GLUT_REDISPLAY_WORK) {
-
+      if (workMask & (GLUT_REDISPLAY_WORK | GLUT_REPAIR_WORK)) {
+        if (__glutMesaSwapHackSupport) {
+          if (window->usedSwapBuffers) {
+            if ((workMask & (GLUT_REPAIR_WORK | GLUT_REDISPLAY_WORK)) == GLUT_REPAIR_WORK) {
+              glXSwapBuffers(__glutDisplay, window->win);
+              goto skippedDisplayCallback1;
+            }
+          }
+        }
         /* Render to normal plane. */
         window->renderWin = window->win;
         window->renderCtx = window->ctx;
-        __glutWindowDamaged = window->damaged;
-        window->damaged = 0;
+        __glutWindowDamaged = (workMask & GLUT_REPAIR_WORK);
         __glutSetWindow(window);
+        window->usedSwapBuffers = 0;
         window->display();
         __glutWindowDamaged = 0;
+
+      skippedDisplayCallback1:;
       }
-      if (workMask & GLUT_OVERLAY_REDISPLAY_WORK) {
+      if (workMask & (GLUT_OVERLAY_REDISPLAY_WORK | GLUT_OVERLAY_REPAIR_WORK)) {
         window = __glutWindowList[num];
         if (window && window->overlay &&
           window->overlay->win == xid && window->overlay->display) {
@@ -896,8 +950,7 @@ processWindowWorkList(GLUTwindow * window)
           /* Render to overlay. */
           window->renderWin = window->overlay->win;
           window->renderCtx = window->overlay->ctx;
-          __glutWindowDamaged = window->overlay->damaged;
-          window->overlay->damaged = 0;
+          __glutWindowDamaged = (workMask & GLUT_OVERLAY_REPAIR_WORK);
           __glutSetWindow(window);
           window->overlay->display();
           __glutWindowDamaged = 0;
@@ -908,15 +961,22 @@ processWindowWorkList(GLUTwindow * window)
         }
       }
     } else {
-      __glutWindowDamaged = window->damaged;
-      window->damaged = 0;
-      if (window->overlay) {
-        __glutWindowDamaged |= window->overlay->damaged;
-        window->overlay->damaged = 0;
+      if (__glutMesaSwapHackSupport) {
+        if (!window->overlay && window->usedSwapBuffers) {
+          if ((workMask & (GLUT_REPAIR_WORK | GLUT_REDISPLAY_WORK)) == GLUT_REPAIR_WORK) {
+            glXSwapBuffers(__glutDisplay, window->win);
+            goto skippedDisplayCallback2;
+          }
+        }
       }
+      /* Render to normal plane (and possibly overlay). */
+      __glutWindowDamaged = (workMask & (GLUT_OVERLAY_REPAIR_WORK | GLUT_REPAIR_WORK));
       __glutSetWindow(window);
+      window->usedSwapBuffers = 0;
       window->display();
       __glutWindowDamaged = 0;
+
+    skippedDisplayCallback2:;
     }
   }
   /* Combine workMask with window->workMask to determine what
@@ -973,7 +1033,7 @@ glutMainLoop(void)
       if (__glutTimerList) {
         waitForSomething();
       } else {
-        processEvents();
+        processEventsAndTimeouts();
       }
     }
   }
