@@ -7,6 +7,15 @@
 
 #include <X11/Xlib.h>
 
+/* SGI optimization introduced in IRIX 6.3 to avoid X server
+   round trips for interning common X atoms. */
+#include <X11/Xatom.h>
+#if defined(_SGI_EXTRA_PREDEFINES) && !defined(NO_FAST_ATOMS)
+#include <X11/SGIFastAtom.h>
+#else
+#define XSGIFastInternAtom(dpy,string,fast_name,how) XInternAtom(dpy,string,how)
+#endif
+
 #include "glutint.h"
 
 /* CENTRY */
@@ -14,9 +23,10 @@ void
 glutFullScreen(void)
 {
   if (__glutMotifHints == None) {
-    __glutMotifHints = XInternAtom(__glutDisplay, "_MOTIF_WM_HINTS", 0);
+    __glutMotifHints = XSGIFastInternAtom(__glutDisplay, "_MOTIF_WM_HINTS",
+      SGI_XA__MOTIF_WM_HINTS, 0);
     if (__glutMotifHints == None) {
-      __glutWarning("Could not intern X atom for WM_COLORMAP_WINDOWS.");
+      __glutWarning("Could not intern X atom for _MOTIF_WM_HINTS.");
     }
   }
   __glutCurrentWindow->desiredX = 0;
