@@ -16,6 +16,15 @@
 #else
 #include <X11/Xmu/StdCmap.h>  /* for XmuLookupStandardColormap */
 #endif
+
+/* SGI optimization introduced in IRIX 6.3 to avoid X server
+   round trips for interning common X atoms. */
+#if defined(_SGI_EXTRA_PREDEFINES) && !defined(NO_FAST_ATOMS)
+#include <X11/SGIFastAtom.h>
+#else
+#define XSGIFastInternAtom(dpy,string,fast_name,how) XInternAtom(dpy,string,how)
+#endif
+
 #include <GL/glut.h>
 #include "glutint.h"
 
@@ -157,12 +166,12 @@ glutSetWindow(int win)
   GLUTwindow *window;
 
   if (win < 1 || win > __glutWindowListSize) {
-    __glutWarning("glutWindowSet attempted on bogus window.");
+    __glutWarning("glutSetWindow attempted on bogus window.");
     return;
   }
   window = __glutWindowList[win - 1];
   if (!window) {
-    __glutWarning("glutWindowSet attempted on bogus window.");
+    __glutWarning("glutSetWindow attempted on bogus window.");
     return;
   }
   __glutSetWindow(window);
@@ -325,12 +334,12 @@ __glutGetVisualInfo(unsigned int mode)
 XVisualInfo *
 __glutDetermineVisual(
   unsigned int displayMode,
-  Bool * singleFake,
+  Bool * treatAsSingle,
   XVisualInfo * (getVisualInfo) (unsigned int))
 {
   XVisualInfo *vis;
 
-  *singleFake = False;
+  *treatAsSingle = GLUT_WIND_IS_SINGLE(displayMode);
   vis = getVisualInfo(displayMode);
   if (!vis) {
     /* Fallback cases when can't get exactly what was asked
@@ -343,7 +352,7 @@ __glutDetermineVisual(
          buffers as no-ops. */
       displayMode |= GLUT_DOUBLE;
       vis = getVisualInfo(displayMode);
-      *singleFake = True;
+      *treatAsSingle = True;
     }
     if (!vis && GLUT_WIND_IS_MULTISAMPLE(displayMode)) {
       /* If we can't seem to get multisampling (ie, not Reality
@@ -400,7 +409,7 @@ __glutSetupColormap(XVisualInfo * vi, GLUTcolormap ** colormap, Colormap * cmap)
     *colormap = NULL;   /* NULL if RGBA */
 #ifndef SOLARIS_2_4_BUG
     /* Solaris 2.4 has a bug in its XmuLookupStandardColormap
-       implementation.  Please compile your Solaris 2.4 version 
+       implementation.  Please compile your Solaris 2.4 version
        of GLUT with -DSOLARIS_2_4_BUG to work around this bug.
        The symptom of the bug is that programs will get a
        BadMatch error from X_CreateWindow when creating a GLUT
@@ -454,8 +463,8 @@ __glutDefaultDisplay(void)
   /* XXX Remove the warning after GLUT 3.0. */
   __glutWarning("The following is a new check for GLUT 3.0; update your code.");
   __glutFatalError(
-    "display needed for window %d, but no display callback.",
-    __glutCurrentWindow->num);
+    "redisplay needed for window %d, but no display callback.",
+    __glutCurrentWindow->num + 1);
 }
 
 void
@@ -473,8 +482,7 @@ __glutDefaultReshape(int width, int height)
     glXMakeCurrent(__glutDisplay, overlay->win, overlay->ctx);
     glViewport(0, 0, (GLsizei) width, (GLsizei) height);
   }
-  /* Make sure we are current to the current layer (application 
-
+  /* Make sure we are current to the current layer (application
      should be able to count on the current layer not changing
      unless the application explicitly calls glutUseLayer). */
   glXMakeCurrent(__glutDisplay, __glutCurrentWindow->renderWin,
@@ -500,7 +508,7 @@ __glutCreateWindow(GLUTwindow * parent,
   window->num = winnum;
 
   window->vis = __glutDetermineVisual(__glutDisplayMode,
-    &window->fakeSingle, __glutGetVisualInfo);
+    &window->treatAsSingle, __glutGetVisualInfo);
   if (!window->vis) {
     __glutFatalError(
       "visual with necessary capabilities not found.");
@@ -553,6 +561,7 @@ __glutCreateWindow(GLUTwindow * parent,
   window->reshape = __glutDefaultReshape;
   window->mouse = NULL;
   window->motion = NULL;
+  window->windowStatus = NULL;
   window->visibility = NULL;
   window->passive = NULL;
   window->entry = NULL;
@@ -572,7 +581,6 @@ __glutCreateWindow(GLUTwindow * parent,
                              VisibilityPartiallyObscured, or
                              VisibilityFullyObscured */
   window->entryState = -1;  /* not EnterNotify or LeaveNotify */
-  window->damaged = 0;
   window->workMask = GLUT_MAP_WORK;
   window->desiredMapState = NormalState;
   window->desiredConfMask = 0;
@@ -585,7 +593,17 @@ __glutCreateWindow(GLUTwindow * parent,
   }
   __glutWindowList[winnum] = window;
   __glutSetWindow(window);
-  if (window->fakeSingle) {
+
+  __glutDetermineMesaSwapHackSupport();
+
+  if (window->treatAsSingle) {
+    /* We do this because either the window really is single
+       buffered (in which case this is redundant, but harmless,
+       because this is the initial single-buffered context
+       state); or we are treating a double buffered window as a
+       single-buffered window because the system does not
+       appear to export any suitable single- buffered visuals
+       (in which the following are necessary). */
     glDrawBuffer(GL_FRONT);
     glReadBuffer(GL_FRONT);
   }
@@ -657,8 +675,8 @@ __glutEstablishColormapsProperty(GLUTwindow * window)
   num = findColormaps(window, winlist, cmaplist, 0, maxcmaps);
   if (num < 2) {
     /* Property no longer needed; remove it. */
-    wmColormapWindows = XInternAtom(__glutDisplay,
-      "WM_COLORMAP_WINDOWS", False);
+    wmColormapWindows = XSGIFastInternAtom(__glutDisplay,
+      "WM_COLORMAP_WINDOWS", SGI_XA_WM_COLORMAP_WINDOWS, False);
     if (wmColormapWindows == None) {
       __glutWarning("Could not intern X atom for WM_COLORMAP_WINDOWS.");
       return;
@@ -671,7 +689,7 @@ __glutEstablishColormapsProperty(GLUTwindow * window)
        WM_COLORMAP_WINDOWS property cannot be intern'ed.  We
        check to be safe. */
     if (status == False)
-      __glutFatalError("XSetWMColormapWindows returned False");
+      __glutFatalError("XSetWMColormapWindows returned False.");
   }
   /* For portability reasons we don't use alloca for winlist
      and cmaplist, but we could. */
@@ -702,7 +720,7 @@ glutCreateWindow(char *title)
     __glutSizeHints.x, __glutSizeHints.y,
     __glutInitWidth, __glutInitHeight);
   win = window->win;
-  /* setup ICCCM properties */
+  /* Setup ICCCM properties. */
   textprop.value = (unsigned char *) title;
   textprop.encoding = XA_STRING;
   textprop.format = 8;
@@ -712,7 +730,7 @@ glutCreateWindow(char *title)
     __glutIconic ? IconicState : NormalState;
   wmHints->flags = StateHint;
   XSetWMProperties(__glutDisplay, win, &textprop, &textprop,
-  /* only put WM_COMMAND property on first window */
+  /* Only put WM_COMMAND property on first window. */
     firstWindow ? __glutArgv : NULL,
     firstWindow ? __glutArgc : 0,
     &__glutSizeHints, wmHints, NULL);
@@ -806,9 +824,9 @@ glutDestroyWindow(int win)
   }
   /* if not a toplevel window... */
   if (window->parent) {
-    /* destroying subwindows may change colormap requirements;
+    /* Destroying subwindows may change colormap requirements;
        recalculate toplevel window's WM_COLORMAP_WINDOWS
-       property */
+       property. */
     __glutPutOnWorkList(__glutToplevelOf(window->parent),
       GLUT_COLORMAP_WORK);
   }
@@ -821,19 +839,35 @@ glutSwapBuffers(void)
   GLUTwindow *window = __glutCurrentWindow;
 
   if (window->renderWin == window->win) {
-    if (__glutCurrentWindow->fakeSingle) {
+    if (__glutCurrentWindow->treatAsSingle) {
       /* Pretend the double buffered window is single buffered,
-         so treat glutSwapBuffers as a no-op */
+         so treat glutSwapBuffers as a no-op. */
       return;
     }
   } else {
-    if (__glutCurrentWindow->overlay->fakeSingle) {
+    if (__glutCurrentWindow->overlay->treatAsSingle) {
       /* Pretend the double buffered overlay is single
-         buffered,  so treat glutSwapBuffers as a no-op. */
+         buffered, so treat glutSwapBuffers as a no-op. */
       return;
     }
   }
+
+  /* For the MESA_SWAP_HACK. */
+  window->usedSwapBuffers = 1;
+
   glXSwapBuffers(__glutDisplay, __glutCurrentWindow->renderWin);
+
+  /* I considered putting the window being swapped on the
+     GLUT_FINISH_WORK work list because you could call
+     glutSwapBuffers from an idle callback which doesn't call
+     __glutSetWindow which normally adds indirect rendering
+     windows to the GLUT_FINISH_WORK work list.  Not being put
+     on the list could lead to the buffering up of multiple
+     redisplays and buffer swaps and hamper interactivity.  I
+     consider this an application bug due to not using
+     glutPostRedisplay to trigger redraws.  If
+     glutPostRedisplay were used, __glutSetWindow would be
+     called and a glFinish to throttle buffering would occur. */
 }
 /* ENDCENTRY */
 
@@ -841,7 +875,7 @@ void
 __glutChangeWindowEventMask(long eventMask, Bool add)
 {
   if (add) {
-    /* add eventMask to window's event mask */
+    /* Add eventMask to window's event mask. */
     if ((__glutCurrentWindow->eventMask & eventMask) !=
       eventMask) {
       __glutCurrentWindow->eventMask |= eventMask;
@@ -849,7 +883,7 @@ __glutChangeWindowEventMask(long eventMask, Bool add)
         GLUT_EVENT_MASK_WORK);
     }
   } else {
-    /* remove eventMask from window's event mask */
+    /* Remove eventMask from window's event mask. */
     if (__glutCurrentWindow->eventMask & eventMask) {
       __glutCurrentWindow->eventMask &= ~eventMask;
       __glutPutOnWorkList(__glutCurrentWindow,
@@ -888,7 +922,7 @@ glutMouseFunc(GLUTmouseCB mouseFunc)
 {
   if (__glutCurrentWindow->mouse) {
     if (!mouseFunc) {
-      /* previous mouseFunc being disabled */
+      /* Previous mouseFunc being disabled. */
       __glutCurrentWindow->buttonUses--;
       __glutChangeWindowEventMask(
         ButtonPressMask | ButtonReleaseMask,
@@ -896,7 +930,7 @@ glutMouseFunc(GLUTmouseCB mouseFunc)
     }
   } else {
     if (mouseFunc) {
-      /* previously no mouseFunc, new one being installed */
+      /* Previously no mouseFunc, new one being installed. */
       __glutCurrentWindow->buttonUses++;
       __glutChangeWindowEventMask(
         ButtonPressMask | ButtonReleaseMask, True);
@@ -923,7 +957,7 @@ glutMotionFunc(GLUTmotionCB motionFunc)
     }
   } else {
     if (motionFunc) {
-      /* previously no mouseFunc, new one being installed */
+      /* Previously no mouseFunc, new one being installed. */
       __glutCurrentWindow->buttonUses++;
       __glutChangeWindowEventMask(
         ButtonPressMask | ButtonReleaseMask, True);
@@ -963,17 +997,36 @@ glutEntryFunc(GLUTentryCB entryFunc)
 }
 
 void
-glutVisibilityFunc(GLUTvisibilityCB visibilityFunc)
+glutWindowStatusFunc(GLUTwindowStatusCB windowStatusFunc)
 {
   __glutChangeWindowEventMask(VisibilityChangeMask,
-    visibilityFunc != NULL);
-  __glutCurrentWindow->visibility = visibilityFunc;
-  if (!visibilityFunc) {
-    /* make state invalid */
+    windowStatusFunc != NULL);
+  __glutCurrentWindow->windowStatus = windowStatusFunc;
+  if (!windowStatusFunc) {
+    /* Make state invalid. */
     __glutCurrentWindow->visState = -1;
   }
 }
 
+static void
+visibilityHelper(int status)
+{
+  if (status == GLUT_HIDDEN || status == GLUT_FULLY_COVERED)
+    __glutCurrentWindow->visibility(GLUT_NOT_VISIBLE);
+  else
+    __glutCurrentWindow->visibility(GLUT_VISIBLE);
+}
+
+void
+glutVisibilityFunc(GLUTvisibilityCB visibilityFunc)
+{
+  __glutCurrentWindow->visibility = visibilityFunc;
+  if (visibilityFunc)
+    glutWindowStatusFunc(visibilityHelper);
+  else
+    glutWindowStatusFunc(NULL);
+}
+
 void
 glutReshapeFunc(GLUTreshapeCB reshapeFunc)
 {
