@@ -47,7 +47,8 @@ extern int sys$gettim(struct timeval *);
 #define IS_AT_OR_AFTER(t1, t2) ((t2).val >= (t1).val)
 
 #else
-#if defined(SVR4) && !defined(sun)  /* Sun claims SVR4, but wants 2 args. */
+#if defined(SVR4) && !defined(sun)  /* Sun claims SVR4, but
+                                       wants 2 args. */
 #define GETTIMEOFDAY(_x) gettimeofday(_x)
 #else
 #define GETTIMEOFDAY(_x) gettimeofday(_x, (struct timezone*) NULL)
@@ -103,6 +104,8 @@ extern int sys$gettim(struct timeval *);
 #define GLUT_DUMMY_WORK		    (1 << 8)
 #define GLUT_FULL_SCREEN_WORK       (1 << 9)
 #define GLUT_OVERLAY_REDISPLAY_WORK (1 << 10)
+#define GLUT_REPAIR_WORK            (1 << 11)
+#define GLUT_OVERLAY_REPAIR_WORK    (1 << 12)
 /* GLUT callback function types */
 typedef void (*GLUTdisplayCB) (void);
 typedef void (*GLUTreshapeCB) (int, int);
@@ -112,6 +115,7 @@ typedef void (*GLUTmotionCB) (int, int);
 typedef void (*GLUTpassiveCB) (int, int);
 typedef void (*GLUTentryCB) (int);
 typedef void (*GLUTvisibilityCB) (int);
+typedef void (*GLUTwindowStatusCB) (int);
 typedef void (*GLUTidleCB) (void);
 typedef void (*GLUTtimerCB) (int);
 typedef void (*GLUTmenuStateCB) (int);  /* DEPRICATED. */
@@ -135,6 +139,7 @@ typedef void (*GLUTmotionFCB) (int *, int *);
 typedef void (*GLUTpassiveFCB) (int *, int *);
 typedef void (*GLUTentryFCB) (int *);
 typedef void (*GLUTvisibilityFCB) (int *);
+typedef void (*GLUTwindowStatusFCB) (int *);
 typedef void (*GLUTidleFCB) (void);
 typedef void (*GLUTtimerFCB) (int *);
 typedef void (*GLUTmenuStateFCB) (int *);  /* DEPRICATED. */
@@ -149,11 +154,13 @@ typedef void (*GLUTbuttonBoxFCB) (int *, int *);
 typedef void (*GLUTtabletMotionFCB) (int *, int *);
 typedef void (*GLUTtabletButtonFCB) (int *, int *, int *, int *);
 #endif
+
 typedef struct _GLUTcolorcell GLUTcolorcell;
 struct _GLUTcolorcell {
   /* GLUT_RED, GLUT_GREEN, GLUT_BLUE */
   GLfloat component[3];
 };
+
 typedef struct _GLUTcolormap GLUTcolormap;
 struct _GLUTcolormap {
   Visual *visual;       /* visual of the colormap */
@@ -164,6 +171,7 @@ struct _GLUTcolormap {
   GLUTcolorcell *cells; /* array of cells */
   GLUTcolormap *next;   /* next colormap in list */
 };
+
 typedef struct _GLUTwindow GLUTwindow;
 typedef struct _GLUToverlay GLUToverlay;
 struct _GLUTwindow {
@@ -186,7 +194,6 @@ struct _GLUTwindow {
   int visState;         /* visibility state (-1 is unknown) */
   int shownState;       /* if window mapped */
   int entryState;       /* entry state (-1 is unknown) */
-  int damaged;          /* is layer damaged by expose? */
 #define GLUT_MAX_MENUS              3
 
   int menu[GLUT_MAX_MENUS];  /* attatched menu nums */
@@ -195,9 +202,11 @@ struct _GLUTwindow {
   GLUTwindow *children; /* list of children */
   GLUTwindow *siblings; /* list of siblings */
   /* Misc. non-API visible (hidden) state. */
-  Bool fakeSingle;      /* faking single buffer with double */
+  Bool treatAsSingle;   /* treat this window as single-buffered
+                           (it might be "fake" though) */
   Bool forceReshape;    /* force reshape before display */
   Bool isDirect;        /* if direct context */
+  Bool usedSwapBuffers; /* if swap buffers used last display */
   long eventMask;       /* mask of X events selected for */
   int buttonUses;       /* number of button uses, ref cnt */
   int tabletPos[2];     /* tablet position (-1 is invalid) */
@@ -206,8 +215,7 @@ struct _GLUTwindow {
   GLUTwindow *prevWorkWin;  /* link list of windows to work on */
   Bool desiredMapState; /* how to mapped window if on map work
                            list */
-  int desiredConfMask;  /* mask of desired window configuration 
-                         */
+  int desiredConfMask;  /* mask of desired window configuration */
   int desiredX;         /* desired X location */
   int desiredY;         /* desired Y location */
   int desiredWidth;     /* desired window width */
@@ -221,6 +229,7 @@ struct _GLUTwindow {
   GLUTpassiveCB passive;  /* passive motion callback (x,y) */
   GLUTentryCB entry;    /* window entry/exit callback (state) */
   GLUTkeyboardCB keyboard;  /* keyboard callback (ASCII,x,y) */
+  GLUTwindowStatusCB windowStatus;  /* visibility callback */
   GLUTvisibilityCB visibility;  /* visibility callback */
   GLUTspecialCB special;  /* special key callback */
   GLUTbuttonBoxCB buttonBox;  /* button box callback */
@@ -239,6 +248,7 @@ struct _GLUTwindow {
   GLUTpassiveFCB fpassive;  /* Fortran passive callback */
   GLUTentryFCB fentry;  /* Fortran entry callback */
   GLUTkeyboardFCB fkeyboard;  /* Fortran keyboard callback */
+  GLUTwindowStatusFCB fwindowStatus;  /* Fortran visibility callback */
   GLUTvisibilityFCB fvisibility;  /* Fortran visibility
                                      callback */
   GLUTspecialFCB fspecial;  /* special key callback */
@@ -250,14 +260,11 @@ struct _GLUTwindow {
                                        callback */
   GLUTspaceButtonFCB fspaceButton;  /* Spaceball button
                                        callback */
-  GLUTtabletMotionFCB ftabletMotion;  /* tablet motion callback 
-
-                                       */
-  GLUTtabletButtonFCB ftabletButton;  /* tablet button callback 
-
-                                       */
+  GLUTtabletMotionFCB ftabletMotion;  /* tablet motion callback */
+  GLUTtabletButtonFCB ftabletButton;  /* tablet button callback */
 #endif
 };
+
 struct _GLUToverlay {
   Window win;
   GLXContext ctx;
@@ -265,14 +272,14 @@ struct _GLUToverlay {
   Colormap cmap;        /* RGB colormap for window; None if CI */
   GLUTcolormap *colormap;  /* colormap; NULL if RGBA */
   int shownState;       /* if overlay window mapped */
-  Bool fakeSingle;      /* faking single buffer with double */
+  Bool treatAsSingle;   /* treat as single-buffered */
   Bool isDirect;        /* if direct context */
-  int damaged;          /* is layer damaged by expose? */
   int transparentPixel; /* transparent pixel value */
   GLUTdisplayCB display;  /* redraw callback */
   /* Special Fortran display callback unneeded since no
      parameters! */
 };
+
 typedef struct _GLUTstale GLUTstale;
 struct _GLUTstale {
   GLUTwindow *window;
@@ -323,6 +330,7 @@ struct _GLUTmenu {
   GLUTselectFCB fselect;  /* callback function of menu */
 #endif
 };
+
 struct _GLUTmenuItem {
   Window win;           /* InputOnly X window for entry */
   GLUTmenu *menu;       /* menu entry belongs to */
@@ -335,6 +343,7 @@ struct _GLUTmenuItem {
   int pixwidth;         /* width of X window in pixels */
   GLUTmenuItem *next;   /* next menu entry on list for menu */
 };
+
 typedef struct _GLUTtimer GLUTtimer;
 struct _GLUTtimer {
   GLUTtimer *next;      /* list of timers */
@@ -345,11 +354,13 @@ struct _GLUTtimer {
   GLUTtimerFCB ffunc;   /* Fortran timer callback */
 #endif
 };
+
 typedef struct _GLUTeventParser GLUTeventParser;
 struct _GLUTeventParser {
   int (*func) (XEvent *);
   GLUTeventParser *next;
 };
+
 /* Declarations to implement glutFullScreen support with
    mwm/4Dwm. */
 
@@ -365,6 +376,7 @@ typedef struct {
   long decorations;
   long input_mode;
 } MotifWmHints;
+
 /* private variables from glut_event.c */
 extern GLUTwindow *__glutWindowWorkList;
 extern int __glutWindowDamaged;
@@ -410,6 +422,9 @@ extern GLUTwindow *__glutCurrentWindow;
 extern int __glutWindowListSize;
 extern void (*__glutFreeOverlayFunc) (GLUToverlay *);
 
+/* private variables from glut_mesa.c */
+extern int __glutMesaSwapHackSupport;
+
 /* private routines from glut_cindex.c */
 extern GLUTcolormap *__glutAssociateColormap(XVisualInfo * vis);
 extern void __glutFreeColormap(GLUTcolormap *);
@@ -469,4 +484,7 @@ extern int __glutIsSupportedByGLX(char *);
 /* private routines from glut_input.c */
 extern void __glutUpdateInputDeviceMask(GLUTwindow * window);
 
+/* private routines from glut_mesa.c */
+extern void __glutDetermineMesaSwapHackSupport(void);
+
 #endif /* __glutint_h__ */
