@@ -35,6 +35,10 @@
 
 #include "fracviewer.h"
 
+#ifdef WIN32
+#define drand48() (((float) rand())/((float) RAND_MAX))
+#define srand48(x) (srand((x)))
+#endif
 
 typedef enum { NOTALLOWED, MOUNTAIN, TREE, ISLAND, BIGMTN, STEM, LEAF, 
                MOUNTAIN_MAT, WATER_MAT, LEAF_MAT, TREE_MAT, STEMANDLEAVES,
@@ -132,7 +136,7 @@ void InitRandTable(unsigned int seed)
 {
   int i;
 
-  srand48(seed);
+  srand48((long) seed);
   for (i = 0; i < NUMRANDS; i++)
     RandTable[i] = drand48() - 0.5;
 }
@@ -254,7 +258,7 @@ void NewMountain(void)
 /***************************** TREE ****************************/
 /***************************************************************/
 
-unsigned long TreeSeed;   /* for srand48 - remember so we can build "same tree"
+long TreeSeed;   /* for srand48 - remember so we can build "same tree"
                      at a different level */
 
  /*
@@ -262,7 +266,7 @@ unsigned long TreeSeed;   /* for srand48 - remember so we can build "same tree"
   */
 void FractalTree(int level)
 {
-  unsigned long savedseed;  /* need to save seeds while building tree too */
+  long savedseed;  /* need to save seeds while building tree too */
 
   if (level == Level) {
       glPushMatrix();
@@ -276,7 +280,7 @@ void FractalTree(int level)
     glTranslatef(0, 1, 0);
     glScalef(0.7, 0.7, 0.7);
 
-      savedseed = drand48()*ULONG_MAX;    /* recurse on a 3-way branching */
+      savedseed = (long) drand48()*ULONG_MAX;    /* recurse on a 3-way branching */
       glPushMatrix();    
         glRotatef(110 + drand48()*40, 0, 1, 0);
         glRotatef(30 + drand48()*20, 0, 0, 1);
@@ -284,7 +288,7 @@ void FractalTree(int level)
       glPopMatrix();
 
       srand48(savedseed);
-      savedseed = drand48()*ULONG_MAX;
+      savedseed = (long) drand48()*ULONG_MAX;
       glPushMatrix();
         glRotatef(-130 + drand48()*40, 0, 1, 0);
         glRotatef(30 + drand48()*20, 0, 0, 1);
