@@ -54,8 +54,6 @@
 /* end of compilation conditions */
 
 /* start various header files needed */
-#include <GL/gl.h>
-#include <GL/glu.h>
 #include <stdlib.h>
 #include <math.h>
 #define GLUT
@@ -455,17 +453,18 @@ Box(float width, float height, float depth, char solid)
       glVertex3f(-x, -y, -z);
       glVertex3f(-x, y, -z);
       glEnd();
-      if (solid)
+      if (solid) {
         glBegin(GL_TRIANGLES);
-      glNormal3f(0.0, 0.0, 1.0);
-      glVertex3f(0.0, 0.0, z);
-      glVertex3f(-x, y, z);
-      glVertex3f(-x, -y, z);
-      glNormal3f(0.0, 0.0, -1.0);
-      glVertex3f(0.0, 0.0, -z);
-      glVertex3f(-x, -y, -z);
-      glVertex3f(-x, y, -z);
-      glEnd();
+        glNormal3f(0.0, 0.0, 1.0);
+        glVertex3f(0.0, 0.0, z);
+        glVertex3f(-x, y, z);
+        glVertex3f(-x, -y, z);
+        glNormal3f(0.0, 0.0, -1.0);
+        glVertex3f(0.0, 0.0, -z);
+        glVertex3f(-x, -y, -z);
+        glVertex3f(-x, y, -z);
+        glEnd();
+      }
       j = 0;
     } else {
       if (!solid)
@@ -478,17 +477,18 @@ Box(float width, float height, float depth, char solid)
       glVertex3f(-y, -x, -z);
       glVertex3f(-y, x, -z);
       glEnd();
-      if (solid)
+      if (solid) {
         glBegin(GL_TRIANGLES);
-      glNormal3f(0.0, 0.0, 1.0);
-      glVertex3f(0.0, 0.0, z);
-      glVertex3f(-y, x, z);
-      glVertex3f(-y, -x, z);
-      glNormal3f(0.0, 0.0, -1.0);
-      glVertex3f(0.0, 0.0, -z);
-      glVertex3f(-y, -x, -z);
-      glVertex3f(-y, x, -z);
-      glEnd();
+        glNormal3f(0.0, 0.0, 1.0);
+        glVertex3f(0.0, 0.0, z);
+        glVertex3f(-y, x, z);
+        glVertex3f(-y, -x, z);
+        glNormal3f(0.0, 0.0, -1.0);
+        glVertex3f(0.0, 0.0, -z);
+        glVertex3f(-y, -x, -z);
+        glVertex3f(-y, x, -z);
+        glEnd();
+      }
       j = 1;
     }
   }
@@ -1083,7 +1083,6 @@ lighting(void)
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_NORMALIZE);
-  glEnable(GL_FLAT);
   glDepthFunc(GL_LESS);
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 
@@ -1385,6 +1384,7 @@ animation(void)
 
 #ifdef GLUT
 #ifdef GLUT_KEY
+/* ARGSUSED1 */
 void
 keyboard(unsigned char key, int x, int y)
 {
@@ -1616,6 +1616,7 @@ keyboard(unsigned char key, int x, int y)
 #endif
 
 #ifdef GLUT_SPEC
+/* ARGSUSED1 */
 void
 special(int key, int x, int y)
 {
@@ -1680,6 +1681,7 @@ menu_select(int mode)
   }
 }
 
+/* ARGSUSED */
 void
 null_select(int mode)
 {
