@@ -38,7 +38,22 @@
  * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
  */
 #include <math.h>
+#ifdef WIN32
+#include <sys/timeb.h>
+#define gettimeofday(_x, _y)          \
+{                                     \
+  struct timeb _t;                    \
+  ftime(&_t);                         \
+  (_x)->tv_sec = _t.time;             \
+  (_x)->tv_usec = _t.millitm * 1000;  \
+}
+#else
 #include <sys/time.h>
+#endif
+/* Some <math.h> files do not define M_PI... */
+#ifndef M_PI
+#define M_PI 3.14159265
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include "objects.h"
@@ -76,12 +91,8 @@ int doublebuffer = 1;		/* Doublebuffer? */
 
 static int RGBA_SB_attributes = GLUT_SINGLE | GLUT_RGBA | GLUT_DEPTH;
 
-static int RGBA_SB_ST_attributes = GLUT_SINGLE | GLUT_RGBA | GLUT_DEPTH | GLUT_STEREO;
-
 static int RGBA_DB_attributes = GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH;
 
-static int RGBA_DB_ST_attributes = GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_STEREO;
-
 float idmat[4][4] = {
     {1.0, 0.0, 0.0, 0.0},
     {0.0, 1.0, 0.0, 0.0},
@@ -495,6 +506,7 @@ void idle(void)
     glutPostRedisplay();
 }
 
+/* ARGSUSED2 */
 void
 mouse(int b, int s, int x, int y)
 {
@@ -505,6 +517,7 @@ mouse(int b, int s, int x, int y)
    }
 }
 
+/* ARGSUSED1 */
 void
 keyboard(unsigned char c, int x, int y)
 {
@@ -949,7 +962,7 @@ void draw_under_table(void)
 {
     int k, l;
 
-    if(FALSE) glEnable(GL_DEPTH_TEST); else glDisable(GL_DEPTH_TEST);
+    glDisable(GL_DEPTH_TEST);
 
 
     glColor3ub(0,  0,  0);
@@ -966,7 +979,7 @@ void draw_under_table(void)
 	glEnd();
     }
 
-    if(TRUE) glEnable(GL_DEPTH_TEST); else glDisable(GL_DEPTH_TEST);
+    glEnable(GL_DEPTH_TEST); 
 
 }
 
