@@ -276,6 +276,7 @@ initialize(void)
   glShadeModel(GL_FLAT);
 }
 
+/* ARGSUSED1 */
 void
 keyboard(unsigned char c, int x, int y)
 {
@@ -307,6 +308,7 @@ keyboard(unsigned char c, int x, int y)
   glutPostRedisplay();
 }
 
+/* ARGSUSED1 */
 void
 special(int key, int x, int y)
 {
@@ -668,10 +670,10 @@ display(void)
 }
 
 static void
-usage(int argc, char *argv[])
+usage(char *name)
 {
   fprintf(stderr, "\n");
-  fprintf(stderr, "usage: %s [ options ]\n", argv[0]);
+  fprintf(stderr, "usage: %s [ options ]\n", name);
   fprintf(stderr, "\n");
   fprintf(stderr, "    Tests texture environments and internal formats\n");
   fprintf(stderr, "\n");
@@ -693,7 +695,7 @@ main(int argc, char *argv[])
     } else if (!strcmp("-db", argv[i])) {
       doubleBuffered = GL_TRUE;
     } else {
-      usage(argc, argv);
+      usage(argv[0]);
       exit(1);
     }
   }
