@@ -5,9 +5,6 @@
    and is provided without guarantee or warrantee expressed or 
    implied. This program is -not- in the public domain. */
 
-#ifdef __sgi
-#include <malloc.h>
-#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <GL/glut.h>
@@ -45,16 +42,13 @@ display(void)
 int
 main(int argc, char **argv)
 {
-#if defined(__sgi)  && !defined(REDWOOD)
-  /* XXX IRIX 6.0.1 mallopt(M_DEBUG, 1) busted. */
-  mallopt(M_DEBUG, 1);
-#endif
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
   glutInitWindowSize(200, 200);
   glutCreateWindow("Test stroke fonts");
   if (glutGet(GLUT_WINDOW_COLORMAP_SIZE) != 0) {
-    __glutFatalError("FAIL: bad RGBA colormap size");
+    printf("FAIL: bad RGBA colormap size\n");
+    exit(1);
   }
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
